"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Replica = exports.ReplicatedObject = void 0;
const tree_1 = require("./map/tree");
class ReplicatedObject {
    constructor(id, tree, replica) {
        this.id = id;
        this.tree = tree;
        this.replica = replica;
        replica.register(this);
    }
    static fromSerialized(id, obj, replica) {
        if (obj[0] !== 'tree')
            throw new Error('Expected SerializedTreeEntry for a Tree');
        return new ReplicatedObject(id, tree_1.nodeFromEntry(obj, []), replica);
    }
}
exports.ReplicatedObject = ReplicatedObject;
class Replica {
    constructor() {
        this.objects = new Map();
    }
    register(object) {
        const list = this.objects.get(object.id) || [];
        if (list.includes(object))
            return;
        this.objects.set(object.id, [...list, object]);
    }
    unregister(object) {
        const listA = this.objects.get(object.id) || [];
        const listB = listA.filter(o => o !== object);
        if (listA.length === listB.length)
            return;
        if (listB.length) {
            this.objects.set(object.id, listB);
        }
        else {
            this.objects.delete(object.id);
        }
    }
    setState(object, state) {
        // TODO
    }
}
exports.Replica = Replica;
//# sourceMappingURL=base.js.map