"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.withOperations = exports.applyOperations = exports.applyOperation = exports.calculateOperations = void 0;
const tree_1 = require("./tree");
const JSON = require("./json");
function getSetValue(value, path) {
    return { type: 'set', path, value };
}
function calculateChanges(a, b, path) {
    if (!a.isLeaf && !b.isLeaf)
        return calculateOperations(a, b, path);
    if (a.isLeaf && b.isLeaf) {
        const { value: valueA } = a;
        const { value: valueB } = b;
        if (valueA === valueB)
            return [];
        // Not exactly perfect, but okay
        if (JSON.stringify(valueA) === JSON.stringify(valueB))
            return [];
    }
    return [getSetValue(b.value, path)];
}
/** Gives a list of operations to apply to the first tree to get the second tree */
function calculateOperations(a, b, path = []) {
    const operations = [];
    let added = Object.entries(b.value);
    for (const key in a.value) {
        const nodeA = a.getNode(key);
        const nodeB = b.getNode(key);
        if (nodeB) {
            operations.push(...calculateChanges(nodeA, nodeB, [...path, key]));
            added = added.filter(e => e[0] !== key);
        }
        else {
            operations.push({ type: 'remove', path: [...path, key] });
        }
    }
    for (const [keyB, valB] of added) {
        operations.push(getSetValue(valB, [...path, keyB]));
    }
    return operations;
}
exports.calculateOperations = calculateOperations;
const createParent = (path, tree) => tree || new tree_1.Tree({}, path);
function applyOperation(tree, operation) {
    const [prefix, key] = tree_1.splitPath(operation.path);
    switch (operation.type) {
        case 'remove': {
            const parent = tree_1.traversePath(tree, prefix);
            if (parent instanceof tree_1.Tree)
                parent.removeNode(key);
            break;
        }
        case 'set': {
            const parent = tree_1.traversePath(tree, prefix, createParent);
            if (parent instanceof tree_1.Tree)
                parent.set(key, operation.value);
            break;
        }
        default: throw new Error(`Unrecognized operation type '${operation.type}'`);
    }
}
exports.applyOperation = applyOperation;
function applyOperations(tree, operations) {
    for (const operation of operations)
        applyOperation(tree, operation);
}
exports.applyOperations = applyOperations;
function withOperations(tree, operations) {
    const result = tree.clone();
    applyOperations(tree, operations);
    return result;
}
exports.withOperations = withOperations;
//# sourceMappingURL=tree-diff.js.map