"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Tree = exports.traversePath = exports.splitPath = void 0;
function splitPath(path) {
    const { length } = path;
    if (!length)
        throw new Error('Expected path with at least 1 entry');
    return [path.slice(0, length - 1), path[length - 1]];
}
exports.splitPath = splitPath;
function traversePath(tree, path, wrongNode) {
    let current = tree;
    for (let i = 0; i < path.length; i++) {
        const currentPath = path.slice(0, i + 1);
        if (current.isLeaf) {
            current = wrongNode === null || wrongNode === void 0 ? void 0 : wrongNode(currentPath, current);
        }
        if (!(current instanceof Tree))
            return undefined;
        current = current.getNode(path[i]) || (wrongNode === null || wrongNode === void 0 ? void 0 : wrongNode(currentPath));
        if (!current)
            return undefined;
    }
    return current;
}
exports.traversePath = traversePath;
function isParentOf(parent, node) {
    const { path: parentPath } = parent;
    const { path: nodePath } = node;
    if (nodePath.length !== parentPath.length + 1)
        return false;
    for (let i = 0; i < parentPath.length; i++) {
        if (nodePath[i] !== parentPath[i])
            return false;
    }
    return true;
}
function expectParentOf(parent, node) {
    if (isParentOf(parent, node))
        return;
    throw new Error(`Expected node '${parent.path}' to be parent of '${node.path}'`);
}
function deepClone(value) {
    if (Array.isArray(value))
        return value.map(deepClone);
    if (typeof value === 'object') {
        const result = {};
        for (const key in value)
            result[key] = deepClone(value[key]);
        return result;
    }
    return value;
}
class Tree {
    constructor(value, path = []) {
        this.value = value;
        this.path = path;
        this.isLeaf = typeof value !== 'object';
    }
    set(key, value) {
        if (this.isLeaf)
            throw new Error('This node is not an object');
        this.value[key] = value;
    }
    addNode(node) {
        expectParentOf(this, node);
        if (this.isLeaf)
            throw new Error('This node is not an object');
        this.value[node.path[this.path.length]] = node.value;
    }
    getNode(key) {
        if (key in this.value) {
            return new Tree(this.value[key], [...this.path, key]);
        }
        return undefined;
    }
    removeNode(nodeOrKey) {
        if (nodeOrKey instanceof Tree) {
            expectParentOf(this, nodeOrKey);
            delete this.value[nodeOrKey.path[nodeOrKey.path.length - 1]];
        }
        else {
            delete this.value[nodeOrKey];
        }
    }
    clone() {
        return new Tree(deepClone(this.value), this.path);
    }
}
exports.Tree = Tree;
//# sourceMappingURL=tree.js.map