"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.applyOperations = exports.applyOperation = exports.calculateOperations = void 0;
const tree_1 = require("./tree");
function getSetNode(node, path) {
    if (node.isLeaf)
        return { type: 'set-leaf', path, value: node.value };
    return { type: 'set-tree', path, entries: node.toSerializedEntry()[2] };
}
function calculateChanges(a, b, path) {
    if (!a.isLeaf && !b.isLeaf)
        return calculateOperations(a, b, path);
    if (a.isLeaf && b.isLeaf) {
        const { value: valueA } = a;
        const { value: valueB } = b;
        if (valueA === valueB)
            return [];
        // Not exactly perfect, but okay
        if (JSON.stringify(valueA) === JSON.stringify(valueB))
            return [];
    }
    return [getSetNode(b, path)];
}
/** Gives a list of operations to apply to the first tree to get the second tree */
function calculateOperations(a, b, path = []) {
    const operations = [];
    let added = [...b.entries];
    for (const [keyA, nodeA] of a.entries) {
        const nodeB = b.getNode(keyA);
        if (nodeB) {
            operations.push(...calculateChanges(nodeA, nodeB, [...path, keyA]));
            added = added.filter(e => e[0] !== keyA);
        }
        else {
            operations.push({ type: 'remove', path: [...path, keyA] });
        }
    }
    for (const [keyB, nodeB] of added) {
        operations.push(getSetNode(nodeB, [...path, keyB]));
    }
    return operations;
}
exports.calculateOperations = calculateOperations;
const createParent = (path, leaf) => leaf || new tree_1.Tree(path);
function applyOperation(tree, operation) {
    const [prefix, key] = tree_1.splitPath(operation.path);
    switch (operation.type) {
        case 'remove': {
            const parent = tree_1.traversePath(tree, prefix);
            if (parent instanceof tree_1.Tree)
                parent.removeNode(key);
            break;
        }
        case 'set-leaf': {
            const parent = tree_1.traversePath(tree, prefix, createParent);
            if (parent instanceof tree_1.Tree)
                parent.addNode(new tree_1.Leaf(operation.path, operation.value));
            break;
        }
        case 'set-tree': {
            const parent = tree_1.traversePath(tree, prefix, createParent);
            if (parent instanceof tree_1.Tree)
                parent.addNode(tree_1.treeFromEntries(operation.entries, operation.path));
            break;
        }
        default: throw new Error(`Unrecognized operation type '${operation.type}'`);
    }
}
exports.applyOperation = applyOperation;
function applyOperations(tree, operations) {
    const result = tree.clone();
    for (const operation of operations)
        applyOperation(result, operation);
    return result;
}
exports.applyOperations = applyOperations;
//# sourceMappingURL=tree-diff.js.map