import { Key } from './operations';
declare type SerializedTreeEntryTree = [type: 'tree', key: Key, entries: SerializedTreeEntry[]];
declare type SerializedTreeEntryLeaf = [type: 'leaf', key: Key, value: any];
export declare type SerializedTreeEntry = SerializedTreeEntryTree | SerializedTreeEntryLeaf;
export declare function nodeFromEntry(entry: SerializedTreeEntry, path: Key[]): Node;
export declare function treeFromEntries(entries: SerializedTreeEntry[], path?: Key[]): Tree;
export declare function treeFromJsObject(obj: object, path?: Key[]): Tree;
export declare function nodeToJsValue(node: Node): any;
export declare function splitPath(path: Key[]): [Key[], Key];
export declare function traversePath(tree: Tree, path: Key[], wrongNode?: (path: Key[], leaf?: Leaf) => Node | undefined): Node | undefined;
export declare abstract class Node {
    readonly isLeaf: boolean;
    readonly path: Key[];
    constructor(isLeaf: boolean, path: Key[]);
    abstract toSerializedEntry(): SerializedTreeEntry;
    abstract clone(): Node;
}
export declare class Tree extends Node {
    entries: Map<Key, Node>;
    constructor(path?: Key[], entries?: ReadonlyMap<Key, Node> | readonly (readonly [Key, Node])[]);
    addNode(node: Node): void;
    getNode(key: Key): Node | undefined;
    removeNode(nodeOrKey: Node | Key): void;
    toSerializedEntry(): SerializedTreeEntry;
    clone(): Tree;
}
export declare class Leaf extends Node {
    readonly value: any;
    constructor(path: Key[], value: any);
    toSerializedEntry(): SerializedTreeEntry;
    clone(): Leaf;
}
export {};
